#ifndef ZP_ASSEMBLY_GENERATOR_H_
#define ZP_ASSEMBLY_GENERATOR_H_

#include "Optimizer.h"
#include <iostream>
#include <string>

namespace zp
{
    class X64AssemblyHolder;
    class X64TextAssemblyHolder;

    class IRVisitor;

    class AssemblyGenerator;
}


class zp::IRVisitor
{
public:
    virtual void visitBlockInstruction(const BlockInstruction&) = 0;
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&) = 0;
    virtual void visitIOInstruction(const IOInstruction&) = 0;
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&) = 0;
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&) = 0;
    virtual void visitLoopInstruction(const LoopInstruction&) = 0;
};


class zp::AssemblyGenerator :
    public IRVisitor
{
    std::ostream& out;

    int labelcount = 0;
    static const char* dataRegister;
    static const char* pointerRegister;
    static const char* argumentRegister1;
    static const char* argumentRegister2;
public:

    AssemblyGenerator(std::ostream& out);

    void finish(void);

    std::string generateLabel(void);
    std::string generateOffset(const std::string& offset, const std::string& base);
    std::string ptr(void);

    virtual void visitBlockInstruction(const BlockInstruction&);
    virtual void visitUnionBlockInstruction(const UnionBlockInstruction&);
    virtual void visitIOInstruction(const IOInstruction&);
    virtual void visitSimpleBlockInstruction(const SimpleBlockInstruction&);
    virtual void visitLinearLoopInstruction(const LinearLoopInstruction&);
    virtual void visitLoopInstruction(const LoopInstruction&);
};




#endif // ZP_ASSEMBLY_GENERATOR_H_
